/**
 * PDF Wrapper class with simple navigation
 * @author Michael Reinhardt
 * @copyright metaSEC e.K.
 */

 window.metaPDF = (function(obj)
 {
     obj.currentPage = 1;
     obj.pages;
 
     obj.doc=null;
     obj.rendering=false;
     obj.pagePending = null;
     obj.scale = 3;
     obj.targetElement="#pdfCanvas";
     obj.canvas = () => j.query(metaPDF.targetElement);
     obj.ctx = () => metaPDF.canvas().getContext('2d');
 
     obj.renderPage = (num,callback) =>
     {
         metaPDF.rendering=true;
         metaPDF.doc.getPage(num).then((page)=>{
             let viewport = page.getViewport({scale:metaPDF.scale * metaPDF.canvas().width / page.getViewport({scale:1.0}).width});
             metaPDF.canvas().height = viewport.height;
             metaPDF.canvas().width = viewport.width;
 
             const renderContext = {
                 canvasContext: metaPDF.ctx(),
                 viewport:viewport
             };
 
             renderTask = page.render(renderContext);
 
             renderTask.promise.then(()=>{
                 metaPDF.rendering=false;
                 if(metaPDF.pagePending !== null)
                 {
                     metaPDF.renderPage(metaPDF.pagePending);
                     metaPDF.pagePending = null;
                 }
                 else
                 {
                     metaPDF.currentPage = num;
                 }
                 if(typeof callback !== "undefined") callback();
             });
         });
     }
 
     obj.hasNext = () =>
     {
         if(metaPDF.openFailed) return false;
         return metaPDF.pages>metaPDF.currentPage;
     }
 
     obj.hasPrevious = () =>
     {
         if(metaPDF.openFailed) return false;
         return metaPDF.currentPage>1;
     }
 
     obj.next = (callback) =>
     {
         if(!metaPDF.hasNext()) return;
         if(metaPDF.rendering) metaPDF.pagePending=metaPDF.currentPage+1;
         else metaPDF.renderPage(metaPDF.currentPage+1,callback);
     };
     obj.back = (callback) =>
     {
         if(!metaPDF.hasPrevious()) return;
         if(metaPDF.rendering) metaPDF.pagePending=metaPDF.currentPage-1;
         else metaPDF.renderPage(metaPDF.currentPage-1,callback);
     };
 
     obj.openFailed=false;
     obj.open = async (url,targetElement,callback,errorCallback) =>
     {
         if(typeof targetElement !== "undefined") metaPDF.targetElement = targetElement;
 
         metaPDF.canvas().style.width='100%';
         metaPDF.canvas().style.height='100%';
         metaPDF.canvas().width=metaPDF.canvas().offsetWidth;
         metaPDF.canvas().height=metaPDF.canvas().offsetHeight;
 
         let promise = pdfjsLib.getDocument(url).promise
         promise.then((pdfDoc) =>
         {
             metaPDF.doc = pdfDoc;
             metaPDF.pages = metaPDF.doc.numPages;
             metaPDF.renderPage(1);
             if(typeof callback !== "undefined") callback();
         },
         (reason) =>
         {
             metaPDF.openFailed=true;
             if(typeof errorCallback !== "undefined") errorCallback(reason);
         });
         await promise;
     };
 
     return obj;
 })({});