/**
 * PDF Wrapper class with simple navigation
 * @author Michael Reinhardt
 * @copyright metaSEC e.K.
 */

if(!window.metaPDF) window.metaPDF = (function(obj)
{
    pdfjsLib.GlobalWorkerOptions.workerSrc = '/jamescore/jslibs/pdfjs-2.8.335-dist/build/pdf.worker.js';

    obj.currentPage = 1;
    obj.pages;

    obj.documents = {};
    obj.currentURI = "";

    obj.rendering=false;
    obj.pagePending = null;
    obj.scale = 1;
    obj.targetElement="#pdfCanvas";
    obj.canvas = () => j.query(metaPDF.targetElement);
    obj.ctx = () => metaPDF.canvas().getContext('2d');

    obj.renderPage = (num,callback) =>
    {
        metaPDF.rendering=true;
        metaPDF.documents[metaPDF.currentURI].getPage(num).then((page)=>{
            let viewport = page.getViewport({scale:metaPDF.scale * metaPDF.canvas().width / page.getViewport({scale:1.0}).width});
            metaPDF.canvas().height = viewport.height;
            metaPDF.canvas().width = viewport.width;

            const renderContext = {
                canvasContext: metaPDF.ctx(),
                viewport:viewport
            };

            renderTask = page.render(renderContext);

            renderTask.promise.then(()=>{
                metaPDF.rendering=false;
                if(metaPDF.pagePending !== null)
                {
                    metaPDF.renderPage(metaPDF.pagePending);
                    metaPDF.pagePending = null;
                }
                else
                {
                    metaPDF.currentPage = num;
                }
                if(typeof callback !== "undefined") callback();
                // page.cleanup();
            });
        });
    }

    obj.hasNext = () =>
    {
        if(metaPDF.openFailed) return false;
        return metaPDF.pages>metaPDF.currentPage;
    }

    obj.hasPrevious = () =>
    {
        if(metaPDF.openFailed) return false;
        return metaPDF.currentPage>1;
    }

    obj.next = (callback) =>
    {
        if(!metaPDF.hasNext()) return;
        if(metaPDF.rendering) metaPDF.pagePending=metaPDF.currentPage+1;
        else metaPDF.renderPage(metaPDF.currentPage+1,callback);
    };
    obj.back = (callback) =>
    {
        if(!metaPDF.hasPrevious()) return;
        if(metaPDF.rendering) metaPDF.pagePending=metaPDF.currentPage-1;
        else metaPDF.renderPage(metaPDF.currentPage-1,callback);
    };

    obj.openFailed=false;
    obj.open = async (url,targetElement,callback,errorCallback) =>
    {
        if(typeof targetElement !== "undefined") metaPDF.targetElement = targetElement;

        metaPDF.canvas().style.width='100%';
        metaPDF.canvas().style.height='100%';
        metaPDF.canvas().width=metaPDF.canvas().offsetWidth;
        metaPDF.canvas().height=metaPDF.canvas().offsetHeight;

        metaPDF.currentPage = 1;
        metaPDF.pages=0;
        metaPDF.rendering=false;
        metaPDF.pagePending = null;

        metaPDF.currentURI=url;

        if(typeof metaPDF.documents[metaPDF.currentURI] !== "undefined")
        {
            metaPDF.pages = metaPDF.documents[metaPDF.currentURI].numPages;
            metaPDF.renderPage(1);
            if(typeof callback !== "undefined") callback();
            return;
        }

        let promise = pdfjsLib.getDocument(metaPDF.currentURI).promise
        promise.then((pdfDoc) =>
        {
            metaPDF.documents[metaPDF.currentURI] = pdfDoc;
            metaPDF.pages = metaPDF.documents[metaPDF.currentURI].numPages;
            metaPDF.renderPage(1);
            if(typeof callback !== "undefined") callback();
        },
        (reason) =>
        {
            metaPDF.openFailed=true;
            if(typeof errorCallback !== "undefined") errorCallback(reason);
        });
        await promise;
    };

    obj.isOpen = (url) =>
    {
        if(typeof url === "undefined")
        {
            return metaPDF.documents[metaPDF.currentURI]!==null;
        }
        else return typeof metaPDF.documents[url] !== "undefined";
    }

    obj.cleanup = async () =>
    {
        if(metaPDF.rendering!==false)
        { console.error("PDF-Lib cleanup cancelled - Rendering in Progress"); return; }
        for(let uri of Object.keys(metaPDF.documents))
        {
            await metaPDF.documents[uri].cleanup();
        }
        if(metaPDF.documents[metaPDF.currentURI]!=null) await metaPDF.documents[metaPDF.currentURI].cleanup();
    }

    obj.closeAll = async () =>
    {
        for(let uri of Object.keys(metaPDF.documents))
        {
            await metaPDF.documents[uri].cleanup();
            await metaPDF.documents[uri].destroy();
            delete metaPDF.documents[uri];
        }
        if(metaPDF.documents[metaPDF.currentURI]!==null)
        {
            await metaPDF.documents[metaPDF.currentURI].cleanup();
            await metaPDF.documents[metaPDF.currentURI].destroy();
            metaPDF.documents[metaPDF.currentURI]=null;
        }
        metaPDF.currentPage = 1;
        metaPDF.pages=0;
        metaPDF.rendering=false;
        metaPDF.pagePending = null;
        metaPDF.scale = 3;
        metaPDF.targetElement="#pdfCanvas";
    }

    return obj;
})({});